<?php
/**
 * Main application class
 *
 * @package wp-es survey-master
 * @author InstaProduct
 * @link http://insta-product.com/
 */

//
//Include all function files
require_once( ES_SURVEY_LIB . '/db.php' );
require_once( ES_SURVEY_LIB . '/list.php' );
require_once( ES_SURVEY_LIB . '/shortcodes.php' );
require_once( ES_SURVEY_APP . '/controller.php' );
require_once( ES_SURVEY_APP . '/viewer.php' );
require_once( ES_SURVEY_LIB . '/list/listing.php' );
require_once( ES_SURVEY_APP . '/front.php' );
require_once( ES_SURVEY_APP . '/front/front-form.php' );
require_once( ES_SURVEY_APP . '/front/QuestionElement.php' );


if ( ! class_exists( 'ES_SURVEY_App' ))
{
	class ES_SURVEY_App
	{
		//{{ run

		/**
		 * Run all functions
		 *
		 * @return void
		 */
		public static function run()
		{
			$hook = sanitize_title( ES_SURVEY_TITLE );

			//Install Plugin
			register_activation_hook( __FILE__, array( __CLASS__, 'install' ) );

			//Hook for plugin loaded
			add_action( 'plugins_loaded', array( __CLASS__, 'updateDb' ) );

			//
			//Admin init
			add_action( 'admin_init', array( __CLASS__, 'adminInit' ) );

			//
			//Register post type
			add_action( 'init', array( __CLASS__, 'registerPostType' ) );

			//
			//Register admin menu
			add_action( 'admin_menu', array( __CLASS__, 'addAdminMenu' ) );
			//
			//Rewrite Handler
			//
			add_action( 'init', array( __CLASS__, 'rewriteFlush' ) );

			//Add css
			add_action(
				'admin_print_styles-toplevel_page_' . ES_SURVEY_PAGENAME,
				array( __CLASS__, 'adminStyle' ),
				1100
			);
			add_action(
				'admin_print_styles-'.$hook.'_page_' . ES_SURVEY_PAGENAME . '-create',
				array( __CLASS__, 'adminStyle' ),
				1100
			);
			add_action(
				'admin_print_styles-'.$hook.'_page_' . ES_SURVEY_PAGENAME . '-setting',
				array( __CLASS__, 'adminStyle' ),
				1100
			);
			//
			//Add javascript
			add_action(
				'admin_print_scripts-toplevel_page_' . ES_SURVEY_PAGENAME,
				array( __CLASS__, 'adminScript' ),
				1100
			);
			add_action(
				'admin_print_scripts-'.$hook.'_page_'.ES_SURVEY_PAGENAME. '-create',
				array( __CLASS__, 'adminScript' ),
				1100
			);
			add_action(
				'admin_print_scripts-'.$hook.'_page_'.ES_SURVEY_PAGENAME. '-setting',
				array( __CLASS__, 'adminScript' ),
				1100
			);
			
			//
			//Front style & script
			add_action('wp_head', array( __CLASS__, 'publicStyle' ) );

			if ( ! is_admin() ) {
				add_action('wp_enqueue_scripts', array( __CLASS__, 'publicScript' ), 10 );
			}
			if( !is_admin() ){
			}
		}
		public static function media_selector_print_scripts() {
			
		}


		//}}
		//{{ install

		/**
		 * Install plugin
		 *
		 * @return void
		 */
		public static function install()
		{
			//Install DB
			$db = new ES_SURVEY_Db();
			$db->install();
		}

		//}}
		//{{ updateDb

		/**
		 * Update Db
		 *
		 * @return void
		 */
		public static function updateDb()
		{
			$db = new ES_SURVEY_Db();
			$db->update();
		}

		//}}
		//{{ adminInit

		/**
		 * Admin init
		 *
		 * @return void
		 */
		public static function adminInit()
		{
			/**
			 * Ajax handler
			 **/
			$controller = new ES_SURVEY_Controller();
			$controller->ajax();
		}

		//}}
		//{{ registerPostType

		/**
		 * Register Post Type
		 *
		 * @return void
		 */
		public static function registerPostType()
		{
			$resultPath = 'smart-survey';
			
			if( get_option( '__ES_SURVEY_base_path', 'smart-survey' )) 
				$resultPath = get_option( '__ES_SURVEY_base_path', 'smart-survey' );
				
			$args	= array(
				'public' => true,
				'show_in_menu' => false,
				'rewrite' => array( 'slug' => $resultPath ),
			);
			register_post_type( 'smart-survey', $args );
		}

		//}}
		//{{ addAdminMenu

		/**
		 * Add admin menu
		 *
		 * @return void
		 */
		public static function addAdminMenu()
		{
			add_menu_page(
				ES_SURVEY_TITLE,
				ES_SURVEY_TITLE,
				'edit_posts',
				ES_SURVEY_PAGENAME,
				array( __CLASS__, 'dashboard' ),
				ES_SURVEY_BASE_URL.'/assets/img/left-icon.png'
			);
			add_submenu_page(
				ES_SURVEY_PAGENAME,
				__( 'All ES Surveys', ES_SURVEY_SLUG ),
				__( 'All ES Surveys', ES_SURVEY_SLUG ),
				'edit_posts',
				ES_SURVEY_PAGENAME,
				array( __CLASS__, 'dashboard' )
			);
			add_submenu_page(
				ES_SURVEY_PAGENAME,
				__( 'Add ES Survey', ES_SURVEY_SLUG ),
				__( 'Add ES Survey', ES_SURVEY_SLUG ),
				'edit_posts',
				ES_SURVEY_PAGENAME . '-create',
				array( __CLASS__, 'create' )
			);
			add_submenu_page(
				ES_SURVEY_PAGENAME,
				__( 'Settings', ES_SURVEY_SLUG ),
				__( 'Settings', ES_SURVEY_SLUG ),
				'edit_posts',
				ES_SURVEY_PAGENAME . '-setting',
				array( __CLASS__, 'setting' )
			);
		}

		//}}
		//{{ dashboard

		/**
		 * Manage all listing
		 *
		 * @return void
		 */
		public static function dashboard()
		{
			$viewer = new ES_SURVEY_Viewer();
			$viewer->dashboard();
		}

		//}}
		//{{ create

		/**
		 * Create new list
		 *
		 * @return void
		 */
		public static function create()
		{
			$viewer = new ES_SURVEY_Viewer();
			$viewer->create();
		}

		//}}
		//{{ setting

		/**
		 * Add administrator setting page
		 *
		 * @return void
		 */
		public static function setting()
		{
			$viewer = new ES_SURVEY_Viewer();
			$viewer->setting();
		}

		//}}
		//{{ rewriteInit

		/**
		 * Init rewrite rule
		 *
		 * @return void
		 */
		public static function rewriteInit()
		{}

		//}}
		//{{ rewriteFlush

		/**
		 * Flush rewrite rule
		 *
		 * @return void
		 */
		public static function rewriteFlush()
		{
			$base = get_option( '__ES_SURVEY_base_path', 'smart-survey' );
			$old  = get_option( '__ES_SURVEY_base_path_old' );

			self::rewriteInit();

			if (!$old || $base != $old ) {
				 flush_rewrite_rules();
				 update_option( '__ES_SURVEY_base_path_old', $base );
			}
		}

		//}}
		//{{ adminStyle

		/**
		 * Add Css file
		 *
		 * @return void
		 */
		public static function adminStyle()
		{
			wp_deregister_style( 'smart-survey-admin-style' );
			wp_register_style(
				'smart-survey-admin-style',
				ES_SURVEY_BASE_URL . 'assets/css/admin.css',
				false,
				null
			);
			wp_enqueue_style('smart-survey-admin-style');
			
			wp_deregister_style( 'bootstrap_css' );
			wp_register_style(
				'bootstrap_css',
				ES_SURVEY_BASE_URL . 'assets/bootstrap/css/bootstrap.min.css',
				false,
				null
			);
			wp_enqueue_style('bootstrap_css');
		}

		//}}
		//{{ adminScript

		/**
		 * Add javascript file
		 *
		 * @return void
		 */
		public static function adminScript()
		{
		?>
			<script type="text/javascript">
				var ES_SURVEY_img_url = '<?php echo ES_SURVEY_BASE_URL . 'assets/img/'; ?>',
					ui_base_image	= '<?php echo ES_SURVEY_BASE_URL . 'assets/img/'; ?>',
					ui_message		 =
					{
						delete_confirm_title: '<?php _e( 'Delete Confirmation', ES_SURVEY_SLUG ); ?>',
						delete_confirm_info: '<?php _e( 'Are you sure doing this action?', ES_SURVEY_SLUG ); ?>'
					}
			</script>

			<?php
			wp_enqueue_media();
			wp_deregister_script('jquery-ui');
			wp_register_script(
				'jquery-ui',
				"http" . ($_SERVER['SERVER_PORT'] == 443 ? "s" : "") . "://ajax.googleapis.com/ajax/libs/jqueryui/1.10.4/jquery-ui.min.js",
				false,
				null
			);
			wp_enqueue_script('jquery-ui');

      		wp_deregister_script('smart-survey-admin-script');
			wp_enqueue_script(
				'smart-survey-admin-script',
				ES_SURVEY_BASE_URL . 'assets/js/admin.js',
				array( 'jquery' ),
				1,
				true
			);
			
			wp_deregister_script('smart-survey-admin-customize-script');
			wp_enqueue_script(
				'smart-survey-admin-customize-script',
				ES_SURVEY_BASE_URL . 'assets/js/admin-customize.js',
				array( 'jquery' ),
				1,
				true
			);
			wp_deregister_script('bootstrap-js');
			wp_enqueue_script(
				'bootstrap-js',
				ES_SURVEY_BASE_URL . 'assets/bootstrap/js/bootstrap.min.js',
				array( 'jquery' ),
				1,
				true
			);
		}

		//}}
		//{{ publicStyle

		/**
		 * Print public style
		 *
		 * @return void
		 */
		public static function publicStyle()
		{
			global $post;
			wp_deregister_style( 'smart-survey-public-style' );
			wp_register_style(
				'smart-survey-public-style',
				ES_SURVEY_BASE_URL . 'assets/public/css/display.css',
				false,
				null
			);
			wp_enqueue_style('smart-survey-public-style');
			if ( 1 || $post->post_type == 'smart-survey' && is_single($post->ID) ){
				wp_deregister_style( 'bootstrap_css' );
				wp_register_style(
					'bootstrap_css',
					ES_SURVEY_BASE_URL . 'assets/bootstrap/css/bootstrap.min.css',
					false,
					null
				);
				wp_enqueue_style('bootstrap_css');
			}
				
		}

		//}}
		//{{ publicScript

		/**
		 * Print public script
		 *
		 * @return void
		 */
		public static function publicScript()
		{
			wp_deregister_script('smart-survey-jst');
			wp_enqueue_script(
				'smart-survey-js',
				ES_SURVEY_BASE_URL . 'assets/public/js/smart-survey.js',
				array( 'jquery' ),
				1,
				true
			);
				
		}
		//}}
	}
}