<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class ES_SURVEY_StepProcess{

	/**
	 * ID of processed form
	 */
	var $form_id;

	/**
	 * Is form processing fineshed
	 */
	var $finished = FALSE;


	var $finished_id;


	var $response_id;

	/**
	 * Initializes the Component.
	 * @since 1.0.0
	 */
	public function __construct() {

		if ( is_admin() ) {
			return NULL;
		}
		// Setting up session if not exists
		if ( ! isset( $_SESSION ) )
			session_start();
		
		add_action( 'parse_request', array( $this, 'process_response' ), 99 );
		
		add_action( 'the_post', array( $this, 'add_post_filter' ) ); // Just hooking in at the beginning of a loop
		
		/* Filter the single_template with our custom function*/
		add_filter('single_template', array( __CLASS__ , 'ES_SURVEY_single_template') );

	} // end constructor
	
	public static  function add_dynamic_css( $form_id ) {
		global $wp_query, $post;
		$html_css =  "<style>";
		if( $form_id  )
		{
			$question_heading_size = get_post_meta( $form_id , 'ES_SURVEY_options_' . 'question-heading-size', true );
			$question_heading_color = get_post_meta( $form_id , 'ES_SURVEY_options_' . 'question-heading-color', true );
			$html_css .= '.survey_form_container .survey-element .question{'. ( ($question_heading_size)?'font-size:'.$question_heading_size.'px !important;':'').
			(($question_heading_color)?'color:'.$question_heading_color.' !important;':'')
.'}';
			$question_sub_heading_size = get_post_meta( $form_id , 'ES_SURVEY_options_' . 'question-sub-heading-size', true );
			$question_sub_heading_color = get_post_meta( $form_id , 'ES_SURVEY_options_' . 'question-sub-heading-color', true );
			
			$html_css .=  '.survey_form_container .survey-element .sub_heading{'. (($question_sub_heading_size)?'font-size:'.$question_sub_heading_size.'px !important;'
			:'').
			(($question_sub_heading_color)?'color:'.$question_sub_heading_color.' !important;':'').'}';
			
			$answer_heading_size = get_post_meta( $form_id , 'ES_SURVEY_options_' . 'answer-heading-size', true );
			$answer_heading_color = get_post_meta( $form_id , 'ES_SURVEY_options_' . 'answer-heading-color', true );
			
			$html_css .=  '.survey-element .answer .display_anser label{'. (($answer_heading_size)?'font-size:'.$answer_heading_size.'px !important;':'').
			(($answer_heading_color)?'color:'.$answer_heading_color.' !important;':'')
.'}';
	
			$answer_sub_heading_size = get_post_meta( $form_id , 'ES_SURVEY_options_' . 'answer-sub-heading-size', true );
			$answer_sub_heading_color = get_post_meta( $form_id , 'ES_SURVEY_options_' . 'answer-sub-heading-color', true );
			
			$html_css .=  '.survey-element .answer .display_anser  span{'. (($answer_sub_heading_size)?'font-size:'.$answer_sub_heading_size.'px !important;':'').
			(($answer_sub_heading_color)?'color:'.$answer_sub_heading_color.' !important;':'')
.'}';
			
		}
		$html_css .=  "</style>";
		return $html_css;
	}
	
	public static  function ES_SURVEY_single_template($single) {
		global $wp_query, $post;
		/* Checks for single template by post type */
		if ($post->post_type == 'smart-survey' ){
			
			$locate_template = locate_template("single-smart-survey.php");
			if( file_exists( $locate_template ) )
			{
				return $locate_template;
			} else {
				if(file_exists(ES_SURVEY_APP . '/templates/single-smart-survey.php'))
				return ES_SURVEY_APP . '/templates/single-smart-survey.php';
			}	
		}
		return $single;
	}
	/**
	 * Adding filter for the content to show Survey
	 * @since 1.0.0
	 */
	public function add_post_filter() {

		add_filter( 'the_content', array( $this, 'the_content' ) );
	}

	/**
	 * The filtered content gets a survey
	 *
	 * @param string $content
	 * @return string $content
	 * @since 1.0.0
	 */
	public function the_content( $content ) {

		global $post, $ES_SURVEY_response_errors;
        // If is no questions form > Exit
        if ( 'smart-survey' != $post->post_type )
            return $content;
		// Set global message on top of page
		
		if ( ! empty( $ES_SURVEY_response_errors ) ) {
			$html  = '<div class="questions-element-error">';
			$html .= '<div class="questions-element-error-message"><p>';
			$html .= esc_attr__( 'There are open answers', 'smart-survey-locale' );
			$html .= '</p></div></div>';
			$html  = apply_filters( 'ES_SURVEY_draw_global_error', $html, $this );

			echo $html;
		}
		$content = $this->show_survey( $post->ID );
		remove_filter( 'the_content', array( $this, 'the_content' ) ); // only show once

		return $content;
	}

	/**
	 * Showing form
	 *
	 * @param int $form_id
	 * @return string $survey_html
	 * @since 1.0.0
	 */
	public function show_survey( $form_id ) {
	   if( !is_single() ) return ;
	   $form_id = $this->getSessionFormId($form_id );
	   $checked_restrictions = $this->check_restrictions( $form_id );
	   if ( TRUE === $checked_restrictions ):
			return $this->survey_form( $form_id );
       elseif( TRUE !== $checked_restrictions ):
	     return $checked_restrictions;
	   endif;	       
	}

    /**
     * Survey form
     *
     * Creating form HTML
     *
     * @param int $form_id
     * @return string $html
     * @since 1.0.0
     */
	private function getSessionFormId ( $form_id )
	{
		//echo "<pre>"; print_r($_SESSION);
		if(isset($_SESSION['CONNECT_SURVEY_ID']) && $_SESSION['CONNECT_SURVEY_ID']>0)
		{
			return $_SESSION['CONNECT_SURVEY_ID'];
			//echo "--1111=".$form_id;
		}
		return $form_id ;
	}
	private function resetPostData ( )
	{
		$_POST['ES_SURVEY_next_step'] = '';
		$_POST['ES_SURVEY_actual_step'] = '';
		$_POST['ES_SURVEY_submission_back'] = '';
		unset($_POST['ES_SURVEY_next_step'] );
		unset($_POST['ES_SURVEY_actual_step'] );
		unset($_POST['ES_SURVEY_submission_back'] );
		
	}
	private function survey_form( $form_id ) {
		
		$form_id = $this->getSessionFormId($form_id );
		//echo "--asdasd=".$form_id;
        global $ES_SURVEY_response_errors, $ES_SURVEY_survey_id;
        $ES_SURVEY_survey_id = $form_id;

        $form = new ES_SURVEY_Front_Form( $form_id );

        do_action( 'before_survey_form' );
        if ( array_key_exists( 'ES_SURVEY_next_step', $_POST ) && 0 == count( $ES_SURVEY_response_errors ) ):
            $next_step = (int) $_POST[ 'ES_SURVEY_next_step' ];
        else:
            if ( array_key_exists( 'ES_SURVEY_actual_step', $_POST ) ):
                $next_step = (int) $_POST[ 'ES_SURVEY_actual_step' ];
            else:
                $next_step = 0;
            endif;
        endif;

        if ( array_key_exists( 'ES_SURVEY_submission_back', $_POST ) ):
            $next_step = (int) $_POST[ 'ES_SURVEY_actual_step' ] - 1;
        endif;

        $actual_step = $next_step;

        $html = '<div class="survey_form_container"><form name="questions" id="smart-survey" action="' . $_SERVER[ 'REQUEST_URI' ] . '" method="POST">';
        $html.= '<input type="hidden" name="_wpnonce" value="' .  wp_create_nonce( 'smart-survey-' . $form_id ) . '" />';

        $step_count = $form->get_step_count();

        if(  0 == $step_count ):
			 //$html .= '<div class="questions-element-error-message">There is no question to ask</div>';
		endif;
		$step_counter_display = get_post_meta( $form_id, 'ES_SURVEY_options_' . 'step-counter', true );
		if( $step_counter_display == "true" && 0 != $step_count ):
			
			$html .= '<header style="">
              <ul id="steps_progression" data-no-of-question="2">';
               $width = (100/($step_count+1));
			   for( $step_display = 1; $step_display <= ( $step_count +1 ) ; $step_display++ ):?>
 				<?php if ( $step_display <= $step_count) :             
                	 $html .= '<li class="step step'.$step_display. ( ( $step_display < ($actual_step + 1) )?' selected-done':'' ). ( ( $step_display ==  ($actual_step + 1) )?' selected':'').'" style="width:'.$width.'%;">Step '.$step_display.'</li>';			   
				 else: 
                	$html .= '<li class="step step'.$step_display. ( ( $step_display < ($actual_step + 1) )?' selected-done':'' ). ( ( $step_display ==  ($actual_step + 1) )?' selected':'').'" style="width:'.$width.'%;">Complete</li>';
				endif; ?>	
              <?php endfor;
             $html .= ' </ul>
	        </header>';
            
			
            /*$html .= '<div class="questions-pagination">' . sprintf(
                    __(
                        'Step <span class="questions-highlight-number">%d</span> of <span class="questions-highlight-number">%s</span>',
                        'smart-survey-locale'
                    ), $actual_step + 1, $step_count +1
                ) . '</div>';*/
        endif;
	
        $elements = $this->get_elements( $form_id, $actual_step );
	    if ( is_array( $elements ) && count( $elements ) > 0 ):
            foreach ( $elements AS $element ):
                    $html .= $element->draw();
					if( $step_count > ($next_step+1) )
						$next_step += 1;
            endforeach;
        else:
            return FALSE;
        endif;
		$html .= '<div class="ES_SURVEY_footer">';
		$previous_button_display = get_post_meta( $form_id, 'ES_SURVEY_options_' . 'previous-button', true );
        if ( $previous_button_display == "true" && 0 < $actual_step ):
            $html .= '<input class="survey-button" type="submit" name="ES_SURVEY_submission_back" value="' . __(
                    'Previous Step', 'smart-survey-locale'
                ) . '"> ';
        endif;

        if ( $actual_step == $next_step ):
            $html .= '<input class="survey-button" type="submit" name="ES_SURVEY_submission" value="' . ($form->has_connect_survey== true?'Next Step': __(
                    'Finish Survey', 'smart-survey-locale'
                )) . '">';
        else:
            $html .= '<input class="survey-button" type="submit" name="ES_SURVEY_submission" value="' . __(
                    'Next Step', 'smart-survey-locale'
                ) . '">';
        endif;

        $html .= '<input type="hidden" name="ES_SURVEY_next_step" value="' . $next_step . '" />';
        $html .= '<input type="hidden" name="ES_SURVEY_actual_step" value="' . $actual_step . '" />';
        $html .= '<input type="hidden" name="ES_SURVEY_id" value="' . $form_id . '" />';
		$html .= '</div>';
        $html .= '</form></div>';
		//Add Css
		$html .= $this->add_dynamic_css( $form_id );        
		return $html;
    }
	
   /**
	 * Getting elements of a survey
	 *
	 * @param int $form_id
	 * @param int $step
	 * @return array $elements
	 * @since 1.0.0
	 */
	public function get_elements( $form_id, $step = 0 ) {

		$survey = new ES_SURVEY_Front_Form( $form_id );

		$actual_step = 0;
		$elements = array();
		foreach ( $survey->elements AS $element ):
			$elements[ $actual_step ][ ] = $element;
			$actual_step ++;
		endforeach;
		foreach ( $survey->bucket_elements AS $element ):
			$elements[ $actual_step ][ ] = $element;
			$actual_step ++;
		endforeach;
		if ( $actual_step < $step ) {
			return FALSE;
		}

		return isset($elements[ $step ])?$elements[ $step ] : false;
	}

	/**
	 * Processing entered data
	 * @since 1.0.0
	 */
	public function process_response() {
		global $ES_SURVEY_survey_id;
		// Form ID was posted or die
		if ( ! array_key_exists( 'ES_SURVEY_id', $_POST ) )
			return;
				
        $ES_SURVEY_survey_id = $_POST[ 'ES_SURVEY_id' ];

        // WP Nonce Check
        if( ! wp_verify_nonce( $_POST[ '_wpnonce' ], 'smart-survey-' . $ES_SURVEY_survey_id ) )
            return;
			
		// Survey exists or die
		if ( ! $this->ES_SURVEY_form_exists( $ES_SURVEY_survey_id ) )
			return;
		
		// Checking restrictions
		if ( TRUE !== $this->check_restrictions( $ES_SURVEY_survey_id ) )
		{
			return;
		}
		
		// If session has data, get it!
		if ( isset( $_SESSION[ 'ES_SURVEY_response' ][ $ES_SURVEY_survey_id ] ) )
			$saved_response = $_SESSION[ 'ES_SURVEY_response' ][ $ES_SURVEY_survey_id ];

		do_action( 'ES_SURVEY_before_process_response', $_POST );

		$response       = array();
		$this->finished = FALSE;

		// Getting data of posted step
		$survey_response = array();
		if ( array_key_exists( 'ES_SURVEY_response', $_POST ) ) {
			$survey_response = $_POST[ 'ES_SURVEY_response' ];
		}

		$survey_actual_step = (int) $_POST[ 'ES_SURVEY_actual_step' ];
		
		// Validating response values and setting up error variables
		$this->validate_response( $ES_SURVEY_survey_id, $survey_response, $survey_actual_step );
		
		// Adding / merging Values to response var
		if ( isset( $saved_response ) ):

			// Replacing old values by key
			if ( is_array( $survey_response ) && count( $survey_response ) > 0 ):
				foreach ( $survey_response AS $key => $answer ):
					$saved_response[ $key ] = $this->ES_SURVEY_prepare_post_data( $answer );
				endforeach;
			endif;

			$response = $saved_response;
		else:
			$response = $survey_response;
		endif;

		$response = apply_filters( 'ES_SURVEY_process_response', $response );
		
		// Storing values in Session
		$_SESSION[ 'ES_SURVEY_response' ][ $ES_SURVEY_survey_id ] = $response;
		

		$this->save_response();
		
		do_action( 'ES_SURVEY_after_process_response', $_POST );
	}

	/**
	 * Saving response data
	 * @since 1.0.0
	 */
	private function save_response() {

		global $ES_SURVEY_response_errors, $ES_SURVEY_survey_id;
		
		do_action( 'ES_SURVEY_before_save_response' );
		if ( ! isset( $_SESSION[ 'ES_SURVEY_response' ][ $ES_SURVEY_survey_id ] ) ) {
			return;
		}
		
		if ( (int) $_POST[ 'ES_SURVEY_actual_step' ] == (int) $_POST[ 'ES_SURVEY_next_step' ] && 0 == count( $ES_SURVEY_response_errors ) && ! array_key_exists( 'ES_SURVEY_submission_back', $_POST ) ):

			$response = $_SESSION[ 'ES_SURVEY_response' ][ $ES_SURVEY_survey_id ];
			if ( $this->save_data( $ES_SURVEY_survey_id, apply_filters( 'ES_SURVEY_save_response', $response ) ) ):
				do_action( 'ES_SURVEY_after_save_response' );

				// Unsetting Session, because not needed anymore
				unset( $_SESSION[ 'ES_SURVEY_response' ] );

				$this->finished    = TRUE;
				$this->finished_id = $ES_SURVEY_survey_id;
				//Here write the code o redirect 
				$form = new ES_SURVEY_Front_Form( $ES_SURVEY_survey_id );
				$elements = $this->get_elements( $ES_SURVEY_survey_id, $form->get_step_count()-1 ); 
				//FInd answer_url
				if( is_object($elements[0]) && isset($elements[0]->id))
				{
					$bucket_question_ans_text = $response [$elements[0]->id];
					if( $bucket_question_ans_text!='' && isset($elements[0]->answers) && is_array($elements[0]->answers))
					{
						$answers = $elements[0]->answers;
						foreach($answers as $keya => $keyv)
						{
							if( $keyv->answer == $bucket_question_ans_text && $keyv->survey_connect_id > 0 && $this->ES_SURVEY_form_exists( $keyv->survey_connect_id )):
								//wp_redirect(get_permalink($keyv->survey_connect_id));
								//echo do_shortcode('[ecommerce_smart_survey id='.$keyv->survey_connect_id.']');
								$this->finished    = false;
								$_SESSION['CONNECT_SURVEY_ID'] = $keyv->survey_connect_id;
								$this->resetPostData ( );
								//die('asd');
								//exit;
							elseif( $keyv->answer == $bucket_question_ans_text && $keyv->redirect_url!=''):
								//wp_redirect($keyv->redirect_url);
								session_destroy();
								if( isset($_REQUEST['es_survey_iframe']) && $_REQUEST['es_survey_iframe'] == 1 ) :
									echo '<html><head></head><body><script>window.top.location.href = "'.$keyv->redirect_url.'";</script> 
</body></html>';exit;			else:
									wp_redirect($keyv->redirect_url);
									exit;
								endif;
							endif;
						}
					}
				}
				if($this->finished    == TRUE)
				{
					session_destroy();
				}
				//End write the code o redirect 
			endif;
		endif;		
		//echo "<pre>"; print_r($this);print_r($_SESSION);
	}

	/**
	 * Validating response
	 *
	 * @param int $form_id
	 * @param array $response
	 * @param int $step
	 * @return boolean $validated
	 * @since 1.0.0
	 */
	public function validate_response( $form_id, $response, $step ) {

		global $ES_SURVEY_response_errors;

		if ( array_key_exists( 'ES_SURVEY_submission_back', $_POST ) ) {
			return FALSE;
		}

		if ( empty( $form_id ) ) {
			return NULL;
		}

		if ( empty( $step ) && (int) $step != 0 ) {
			return NULL;
		}

		$elements = $this->get_elements( $form_id, $step );

		if ( ! is_array( $elements ) && count( $elements ) == 0 ) {
			return NULL;
		}

		if ( empty( $ES_SURVEY_response_errors ) ) {
			$ES_SURVEY_response_errors = array();
		}

		// Running true all elements
		foreach ( $elements AS $element ):
			
			$skip_validating = apply_filters( 'ES_SURVEY_skip_validating', FALSE, $element );

			if ( $skip_validating ) {
				continue;
			}

			$answer = '';
			if ( array_key_exists( $element->id, $response ) ) {
				$answer = $response[ $element->id ];
			}

			if ( ! $element->validate( $answer ) ):

				if ( empty( $ES_SURVEY_response_errors[ $element->id ] ) ) {
					$ES_SURVEY_response_errors[ $element->id ] = array();
				}

				// Getting every error of question back
				foreach ( $element->validate_errors AS $error ):
					$ES_SURVEY_response_errors[ $element->id ][ ] = $error;
				endforeach;

			endif;
		endforeach;

		if ( is_array( $ES_SURVEY_response_errors ) && array_key_exists( $element->id, $ES_SURVEY_response_errors ) ):
			// ??? One Element at the end ???
			if ( is_array( $ES_SURVEY_response_errors[ $element->id ] )
				&& count(
					$ES_SURVEY_response_errors[ $element->id ]
				) == 0
			):
				return TRUE;
			else:
				return FALSE;
			endif;
		else:
			return TRUE;
		endif;

	}

	/**
	 * Sub function for save_response
	 *
	 * @param int   $form_id
	 * @param array $response
	 * @return boolean $saved
	 * @since 1.0.0
	 */
	private function save_data( $form_id, $response ) {

		global $wpdb, $ES_SURVEY_global, $current_user;
		
		$responds_table = $wpdb->prefix . 'ES_SURVEY_responds';
		$respond_answers_table = $wpdb->prefix . 'ES_SURVEY_respond_answers';


		wp_get_current_user();
		$user_id = $user_id = $current_user->ID;

		if ( '' == $user_id ) {
			$user_id = - 1;
		}

		// Adding new question
		$wpdb->insert(
			$responds_table,
			array(
				'ES_SURVEY_id' => $form_id,
				'user_id'      => $user_id,
				'timestamp'    => time(),
				'remote_addr'  => $_SERVER[ 'REMOTE_ADDR' ]
			)
		);

		do_action( 'ES_SURVEY_save_data', $form_id, $response );

        $response_id       = $wpdb->insert_id;
		$this->response_id = $response_id;

		foreach ( $response AS $element_id => $answers ):

			if ( is_array( $answers ) ):

				foreach ( $answers AS $answer ):
					$wpdb->insert(
						$respond_answers_table,
						array(
							'respond_id'  => $response_id,
							'question_id' => $element_id,
							'value'       => $answer
						)
					);
				endforeach;

			else:
				$answer = $answers;

				$wpdb->insert(
					$respond_answers_table,
					array(
						'respond_id'  => $response_id,
						'question_id' => $element_id,
						'value'       => $answer
					)
				);

			endif;
		endforeach;

		return TRUE;
	}

	
	/**
	 * Text which will be shown after a user has participated successful
	 *
	 * @param int $form_id
	 * @return string $html
	 * @since 1.0.0
	 */
	public function text_thankyou_for_participation( $form_id ) {

		$show_results = 'no';
		
		$html = '<div id="questions-thank-participation">';
		$html .= '<p>' . __( 'Thank you for participating this survey!', 'smart-survey-locale' ) . '</p>';
		$html .= '</div>';
		return apply_filters( 'ES_SURVEY_text_thankyou_for_participation', $html, $form_id );
	}

	/**
	 * Showing results
	 *
	 * @param int $survey_id
	 * @return string $html
	 * @since 1.0.0
	 */
	public function show_results( $form_id ) {

		$html = '<p>' . __( 'This are the actual results:', 'smart-survey-locale' ) . '</p>';
		$html .= do_shortcode( '[ES_SURVEY_results id="' . $form_id . '"]' );

		return apply_filters( 'ES_SURVEY_show_results', $html, $form_id );
	}
	
	private function check_restrictions( $form_id ) {
		if ( $this->finished && $this->finished_id == $form_id ):
			return $this->text_thankyou_for_participation( $form_id );
		endif;
		return TRUE;
	}
	
	public function ES_SURVEY_form_exists( $form_id ) {

	global $wpdb;

	$sql = $wpdb->prepare( "SELECT COUNT( ID ) FROM {$wpdb->prefix}posts WHERE ID = %d and post_type = 'smart-survey'", $form_id );
	$var = $wpdb->get_var( $sql );

	if ( $var > 0 ) {
		return TRUE;
	}

	return FALSE;
	}
	
	public function ES_SURVEY_prepare_post_data( $data ){
	// Do not preparing objects or arrays	
	if( is_object( $data ) || is_array( $data ) )
		return $data;
	
	$data = trim( $data );
	$data = stripslashes( $data );
	return $data;
}
	
}

global $ES_SURVEY_StepProcess;
$ES_SURVEY_StepProcess = new ES_SURVEY_StepProcess();