<?php
if ( !defined( 'ABSPATH' ) ) exit;

class ES_SURVEY_QuestionElement {

	var $id = NULL;
	var $survey_id;
	var $slug;
	var $title;
	var $description;
	var $icon_url;
	var $question_data;
	var $is_question = TRUE;
	var $has_answers = FALSE;
	var $answers = array();
	var $response;
	var $initialized = FALSE;

	/**
	 * Constructor
	 * @param int $id ID of the element
	 * @since 1.0.0
	 */
	public function __construct( $id = NULL ) {

		$this->slug        = 'Question';
		$this->title       = esc_attr__( 'Add Question', 'smart-survey-locale' );
		$this->description = esc_attr__(
			'Add a question which can be answered by selecting one of the given answers.', 'smart-survey-locale'
		);
		$this->icon_url        = '';

		$this->has_answers  = TRUE;
		
		if ( NULL != $id && '' != $id ) {
			$this->populate( $id );
		}
	}
	

	/**
	 * Populating element object with data
	 * @param int $id Element id
	 * @since 1.0.0
	 */
	private function populate( $id ) {

		global $wpdb, $ES_SURVEY_global;
		
		$questions_table = $wpdb->prefix . 'ES_SURVEY_questions';
		$answers_table = $wpdb->prefix . 'ES_SURVEY_answers';
		
		
		$this->question_data = array();
		$this->answers  = array();

		$sql = $wpdb->prepare( "SELECT * FROM {$questions_table} WHERE id = %s", $id );
		$row = $wpdb->get_row( $sql );
		

		$this->id = $id;
		$this->question_data =  $row ;
		$this->ES_SURVEY_id = $row->ES_SURVEY_id;

		$sql     = $wpdb->prepare(
			"SELECT * FROM {$answers_table} WHERE question_id = %s ORDER BY answer_no ASC", $id
		);
		$results = $wpdb->get_results( $sql );

		if ( is_array( $results ) ):
			foreach ( $results AS $result ):
			{
				if( isset($result) && isset($result->answer) && $result->answer!='' )
				$this->answers[$result->id] = $result ;
			}
			endforeach;
		endif;
	}
	
	
	/**
	 * Validate user input - Have to be overwritten by child classes if element needs validation
	 * @return bool
	 * @since 1.0.0
	 */
	public function validate( $input ) {

		$error = FALSE;

		if ( empty( $input ) ):
			$this->validate_errors[ ] = sprintf( esc_attr__( 'Please select a value.', 'smart-survey-locale' ) );
			$error                    = TRUE;
		endif;

		if ( $error ):
			return FALSE;
		endif;

		return TRUE;
	}

	/**
	 * Drawing Element on frontend
	 * @return string $html Element HTML
	 * @since 1.0.0
	 */
	public function draw() {
	
		global $ES_SURVEY_response_errors;

		$errors = '';
		if ( is_array( $ES_SURVEY_response_errors ) && array_key_exists( $this->id, $ES_SURVEY_response_errors ) ) {
			$errors = $ES_SURVEY_response_errors[ $this->id ];
		}
	
		$html = '';
	
		$html = apply_filters( 'ES_SURVEY_draw_element_outer_start', $html, $this );
	
		$element_classes = array( 'survey-element', 'survey-element-' . $this->id );
		$element_classes = apply_filters( 'ES_SURVEY_element_classes', $element_classes, $this );
	
		$html .= '<div class="' . implode( ' ', $element_classes ) . '">';
	
		$html = apply_filters( 'ES_SURVEY_draw_element_inner_start', $html, $this );
	
		// Echo Errors
		if ( is_array( $errors ) && count( $errors ) > 0 ):
			$html .= '<div class="questions-element-error">';
			$html .= '<div class="questions-element-error-message">';
			$html .= '<ul class="questions-error-messages">';
			foreach ( $errors AS $error ):
				$html .= '<li>' . $error . '</li>';
			endforeach;
			$html = apply_filters( 'ES_SURVEY_draw_element_errors', $html, $this );
			$html .= '</ul></div>';
		endif;
	
		if ( ! empty( $this->question_data->question ) ):
			$html .= '<div class="question">' . $this->question_data->question . '</div>';
		endif;
		
		if ( ! empty( $this->question_data->sub_heading ) ):
			$html .= '<div class="sub_heading">' . $this->question_data->sub_heading . '</div>';
		endif;
		
		// Fetching user response data
		$this->get_response();
	
		$html .= '<div class="answer col-sm-12">';

        if ( 0 == count( $this->answers ) && $this->has_answers == TRUE ) {
            $html.= '<p>' . esc_attr__(
                'You don´t entered any answers. Please add some to display answers here.', 'smart-survey-locale'
            ) . '</p>';
        }else {
            $html .= $this->input_html( $this->question_data->display  );
        }

		$html .= '</div>';
	
		// End Echo Errors
		if ( is_array( $errors ) && count( $errors ) > 0 ):
			$html .= '</div>';
		endif;
	
		$html = apply_filters( 'ES_SURVEY_draw_element_inner_end', $html, $this );
	
		$html .= '</div>';
	
		$html = apply_filters( 'ES_SURVEY_draw_element_outer_end', $html, $this );
	
		return $html;
	}
	
	/**
	 * Contains element HTML on frontend - Have to be overwritten by child classes
	 * @return string $html Element frontend HTML
	 */
	public function input_html( $display = NULL )  {
		
		$main_class ="col-sm-12";
		if(!$display || empty ( $display ) )
		{
			$display = "vertical";;	
		}
		if( strtolower($display) == 'horizontal')
		{
			$display = 'horizontal';
			$main_class="col-sm-2";
		}
		
		$html = '';
		foreach ( $this->answers AS $answer ):
			$html .= '<div class="display_anser '.$main_class.'">';
			$checked = '';
			if ( $this->response == $answer->answer ) {
				$checked = ' checked="checked"';
			}
			$html .= '<label><input type="radio" name="' . $this->get_input_name(
				) . '" value="' . $answer->answer . '" ' . $checked . '/> <strong>' . $answer->answer . '</strong></label>';
			//Display Image
			if ( isset($answer->sub_text) && $answer->sub_text!='' ) {
				$html .= '<span>' . $answer->sub_text . '</span>';
			}
			//Display Image
			if ( isset($answer->image_url) && $answer->image_url!='' ) {
				$html .= '<img class="url_image" src="'.$answer->image_url.' "/>';
			}
			else if ( isset($answer->image_attachment_id) && $answer->image_attachment_id > 0 ) {
				$image_attachment_image = wp_get_attachment_image_src( $answer->image_attachment_id, array(200, 200) );
				if( isset($image_attachment_image[0]) )
				{
					$html .= '<img src="'.$image_attachment_image[0].' "/>';
				}
			}
			$html .= '</div>';
		endforeach;
		return $html;
	}
	/**
	 * Getting posted data from an answering user
	 * @return array $response The post response
	 * @since 1.0.0
	 */
	private function get_response() {
	
		global $ES_SURVEY_survey_id;
	
		$this->response = FALSE;
	
		// Getting value/s
		if ( ! empty( $ES_SURVEY_survey_id ) ):
			if ( isset( $_SESSION[ 'ES_SURVEY_response' ] ) ):
				if ( isset( $_SESSION[ 'ES_SURVEY_response' ][ $ES_SURVEY_survey_id ] ) ):
					if ( isset( $_SESSION[ 'ES_SURVEY_response' ][ $ES_SURVEY_survey_id ][ $this->id ] ) ):
						$this->response = $_SESSION[ 'ES_SURVEY_response' ][ $ES_SURVEY_survey_id ][ $this->id ];
					endif;
				endif;
			endif;
		endif;
	
		return $this->response;
	}
	
	/**
	 * Returns the name of an input element
	 * @return string $input_name The name of the input
	 * @since 1.0.0
	 */
	public function get_input_name() {
		$input_name = 'ES_SURVEY_response[' . $this->id . ']';
		return $input_name;
	}
	
	/**
	 * Get all saved responses of an element
	 * @return mixed $responses The responses as array or FALSE if failed to get responses
	 * @since 1.0.0
	 */
	public function get_responses() {
	
		global $wpdb, $ES_SURVEY_global;
		
		$responds_table = $wpdb->prefix . 'ES_SURVEY_responds';
		$respond_answers_table = $wpdb->prefix . 'ES_SURVEY_respond_answers';


		$sql       = $wpdb->prepare(
			"SELECT * FROM {$responds_table} AS r, {$respond_answers_table} AS a WHERE r.id=a.respond_id AND a.question_id=%d",
			$this->id
		);
		$responses = $wpdb->get_results( $sql );
	
		$result_answers               = array();
		$result_answers[ 'question' ] = $this->question_data->question;
	
		if ( is_array( $this->answers ) && count( $this->answers ) > 0 ):
			// If element has predefined answers
			foreach ( $this->answers AS $answer_id => $answer ):
				$value = FALSE;
				if ( $this->answer_is_multiple ):
					foreach ( $responses AS $response ):
						if ( $answer->answer == $response->value ):
							$result_answers[ 'answer' ][ $response->respond_id ][ $answer->answer ] = esc_attr__(
								'Yes'
							);
						elseif ( ! isset( $result_answers[ 'responses' ][ $response->respond_id ][ $answer->answer ] ) ):
							$result_answers[ 'responses' ][ $response->respond_id ][ $answer->answer ] = esc_attr__(
								'No'
							);
						endif;
					endforeach;
				else:
					foreach ( $responses AS $response ):
						if ( $answer->answer == $response->value ):
							$result_answers[ 'responses' ][ $response->respond_id ] = $response->value;
						endif;
					endforeach;
				endif;
	
			endforeach;
		else:
			// If element has no predefined answers
			if ( is_array( $responses ) && count( $responses ) > 0 ):
				foreach ( $responses AS $response ):
					$result_answers[ 'responses' ][ $response->respond_id ] = $response->value;
				endforeach;
			endif;
		endif;
	
		if ( is_array( $result_answers ) && count( $result_answers ) > 0 ) {
			return $result_answers;
		} else {
			return FALSE;
		}
	}
}