<?php
if ( ! class_exists( 'ES_SURVEY_Db' ))
{
	/**
	 * Class for manage plugin database
	 */
	class ES_SURVEY_Db
	{
		/**
		 * Take db version
		 *
		 * @var string
		 */
		public $version = '1.0';

		/**
		 * Class construct
		 *
		 * @return void
		 */
		public function __construct()
		{
			global $ES_SURVEY_db_version;
			$ES_SURVEY_db_version = $this->version;
		}

		/**
		 * Database query
		 *
		 * @return string
		 */
		public function query_stats()
		{
			global $wpdb;

			
			return false;
		}

		/**
		 * Database query
		 *
		 * @return string
		 */
		public function query()
		{
			$sql  = '';
			$sql .= $this->query_stats();
			return $sql;
		}

		/**
		 * install
		 *
		 * @return void
		 */
		public function install()
		{
			global $wpdb, $ES_SURVEY_db_version;
			$installed_ver = get_option( "ES_SURVEY_db_version" );

			if( ! $this->is_installed() || ! $installed_ver || ( $installed_ver != $ES_SURVEY_db_version ) )
			{
				require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
				
				$table_questions       = $wpdb->prefix . 'ES_SURVEY_questions';
				$table_answers         = $wpdb->prefix . 'ES_SURVEY_answers';
				$table_responds        = $wpdb->prefix . 'ES_SURVEY_responds';
				$table_respond_answers = $wpdb->prefix . 'ES_SURVEY_respond_answers';
				
				$sql = "CREATE TABLE $table_questions (
					id int(11) NOT NULL AUTO_INCREMENT,
					ES_SURVEY_id int(11) DEFAULT NULL,
					question text DEFAULT NULL,
					sub_heading text DEFAULT NULL,
					display char(20) DEFAULT NULL,
					question_no INT(11) DEFAULT NULL,
					UNIQUE KEY id (id)
					) ENGINE = INNODB DEFAULT CHARSET = utf8;";
		
				dbDelta( $sql );
		
				$sql = "CREATE TABLE $table_answers (
					id int(11) NOT NULL AUTO_INCREMENT,
					question_id int(11) DEFAULT NULL,
					answer text DEFAULT NULL,
					sub_text text DEFAULT NULL, 
					image_url varchar(255) DEFAULT NULL, 
					redirect_url text DEFAULT NULL, 
					image_attachment_id int(11) DEFAULT NULL,
					answer_no int(11) DEFAULT NULL,
					survey_connect_id int(11) DEFAULT NULL,
					UNIQUE KEY id (id)
					) ENGINE = INNODB DEFAULT CHARSET = utf8;";
		
				dbDelta( $sql );
		
				$sql = "CREATE TABLE $table_responds (
					id int(11) DEFAULT NULL AUTO_INCREMENT,
					ES_SURVEY_id int(11) DEFAULT NULL,
					user_id int(11) DEFAULT NULL,
					timestamp int(11) DEFAULT NULL,
					remote_addr char(15) NOT NULL,
					cookie_key char(30) DEFAULT NULL,
					UNIQUE KEY id (id)
					) ENGINE = INNODB DEFAULT CHARSET = utf8;";
		
				dbDelta( $sql );
		
				$sql = "CREATE TABLE $table_respond_answers (
					id int(11) DEFAULT NULL AUTO_INCREMENT,
					respond_id int(11) DEFAULT NULL,
					question_id int(11) DEFAULT NULL,
					value text DEFAULT NULL,
					UNIQUE KEY id (id)
					) ENGINE = INNODB DEFAULT CHARSET = utf8;";
		
				dbDelta( $sql );
		
				$sql = "ALTER TABLE $table_questions CONVERT TO CHARACTER SET utf8 collate utf8_general_ci;";
				$wpdb->query( $sql );
		
				$sql = "ALTER TABLE $table_answers CONVERT TO CHARACTER SET utf8 collate utf8_general_ci;";
				$wpdb->query( $sql );
		
				$sql = "ALTER TABLE $table_responds CONVERT TO CHARACTER SET utf8 collate utf8_general_ci;";
				$wpdb->query( $sql );
		
				$sql = "ALTER TABLE $table_respond_answers CONVERT TO CHARACTER SET utf8 collate utf8_general_ci;";
				$wpdb->query( $sql );
		
				update_option( "ES_SURVEY_db_version", $ES_SURVEY_db_version );
				
				update_option( 'ES_SURVEY_is_installed', TRUE );
			}
		}

		/**
		 * update
		 *
		 * @return void
		 */
		public function update()
		{
			global $ES_SURVEY_db_version;
			if ( get_site_option( 'ES_SURVEY_db_version' ) != $ES_SURVEY_db_version ) {
				$this->install();
			}
		}
		
		public static function is_installed(){
			global $wpdb;
			
			$tables = array(		
				$wpdb->prefix . 'ES_SURVEY_questions',
				$wpdb->prefix . 'ES_SURVEY_answers',
				$wpdb->prefix . 'ES_SURVEY_responds',
				$wpdb->prefix . 'ES_SURVEY_respond_answers',
			);
			
			// Checking if all tables are existing
			$not_found = FALSE;
			foreach( $tables AS $table ):
				if( $wpdb->get_var( "SHOW TABLES LIKE '$table'" ) != $table ):
					$not_found = TRUE;
				endif;
			endforeach;
			
			$is_installed_option = (boolean) get_option( 'ES_SURVEY_is_installed', FALSE );
			
			if( $not_found || FALSE == $is_installed_option )
				return FALSE;
			
			return TRUE;
		}
	}

}
?>